/* osgVisual test. distortionNG, experimental.
*
*  Permission is hereby granted, free of charge, to any person obtaining a copy
*  of this software and associated documentation files (the "Software"), to deal
*  in the Software without restriction, including without limitation the rights
*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
*  copies of the Software, and to permit persons to whom the Software is
*  furnished to do so, subject to the following conditions:
*
*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
*  THE SOFTWARE.
*/

#pragma once

#include <osgDB/FileUtils>
#include <osg/Texture2D>

#include "DistortionSetupStrategy.h"

/**
 * \brief This class parses the Projection Dsigner specific configuration files.
 * 
 * @author Torben Dannhauer
 * @date  Aug 2012
 */ 
class CameraConfigParser : public osg::Referenced
{
public:
	CameraConfigParser();
	virtual ~CameraConfigParser() {};

	bool parseConfigFile(const char * filename);

	bool extractFrustum(std::string data_);
	bool extractRotation(std::string data_);
	bool extractTranslation(std::string data_);

	std::vector<double> getFrustumDataset() {return(frustumValues);}	
	std::vector<double> getRotationDataset() {return(rotationValues);}
	std::vector<double> getTranslationDataset() {return(translationValues);}
	bool isConfigParsed() {return configParsed;};

private:
	bool configParsed;
	std::vector<double> frustumValues;
	std::vector<double> rotationValues;
	std::vector<double> translationValues;
};


/**
 * \brief "Projection Designer" Setup Strategy reads setup files of Projection Designer and configures the distortion accordingly.
 * 
 * @author Torben Dannhauer, Ludwig Friedmann
 * @date  Aug 2012
 */ 
class DistortionSetupStrategyProjectionDesigner :
	public DistortionSetupStrategy
{
public:
	DistortionSetupStrategyProjectionDesigner();
	~DistortionSetupStrategyProjectionDesigner();
	void delegateDistortionSetup(osgViewer::DistortionSet* distortionSet);
	void setDistortionInputFiles(std::string distortionFile, std::string blendmapFile, std::string frustumFile);

	std::vector<double> getFrustumDataset() {return parser->getFrustumDataset();};
	
	std::vector<double> getRotationDataset() {return parser->getRotationDataset();};

	std::vector<double> getTranslationDataset() {return parser->getTranslationDataset();};

private:
	bool _distortionInitialized;				//Is Distortion already initialized?
	std::string _blendmapFilename;
	std::string _frustumFilename;
	std::string _distortionFilename;



	osg::Texture* loadTexture( const std::string& fileName );

	osg::ref_ptr<CameraConfigParser> parser;
	osg::ref_ptr<osg::Texture> distortMapTexture;
};



